@@ -19,6 +19,8 @@
 #ifndef __CAIRO_EXCEPTION_H
 #define __CAIRO_EXCEPTION_H
 
+#include <cairommconfig.h>
+
 #include <cairomm/enums.h>
 #include <stdexcept>
 
@@ -35,11 +37,11 @@ namespace Cairo
 class logic_error: public std::logic_error
 {
 public:
-  explicit logic_error(ErrorStatus status);
-  virtual ~logic_error() noexcept;
+  CAIROMM_API explicit logic_error(ErrorStatus status);
+  CAIROMM_API ~logic_error() noexcept override;
 
   //virtual const char* what() const noexcept;
-  ErrorStatus get_status_code() const;
+  CAIROMM_API ErrorStatus get_status_code() const;
 
 private:
   ErrorStatus m_status;
