@@ -25,14 +25,6 @@
 #include <cairomm/types.h>
 #include <cairomm/refptr.h>
 
-/* following is required for OS X */
-
-#ifdef nil
-#undef nil
-#endif
-
-/* end OS X */
-
 #include <cairo.h>
 #ifdef CAIRO_HAS_FT_FONT
 #include <cairo-ft.h>
@@ -42,8 +34,8 @@
 namespace Cairo
 {
 
-class ScaledFont;
-class Context;
+class CAIROMM_API ScaledFont;
+class CAIROMM_API Context;
 
 /**
  * A FontFace represents a particular font at a particular weight, slant, and
@@ -53,7 +45,7 @@ class Context;
  * using the toy text API by way of Context::select_font_face(). The resulting
  * face can be accessed using Context::get_font_face().
  */
-class FontFace
+class CAIROMM_API FontFace
 {
 protected:
 
@@ -66,6 +58,8 @@ public:
    */
   explicit FontFace(cairo_font_face_t* cobject, bool has_reference = false);
 
+  FontFace(const FontFace&) = delete;
+  FontFace& operator=(const FontFace&) = delete;
 
   virtual ~FontFace();
 
@@ -75,6 +69,9 @@ public:
   void set_user_data(const cairo_user_data_key_t *key, void *user_data, cairo_destroy_func_t destroy);
   */
 
+  /**
+   * Returns the type of the backend used to create a font face
+   */
   FontType get_type() const;
 
   typedef cairo_font_face_t cobject;
@@ -104,7 +101,7 @@ protected:
  * A simple font face used for the cairo 'toy' font API.
  * @since 1.8
  */
-class ToyFontFace : public FontFace
+class CAIROMM_API ToyFontFace : public FontFace
 {
 public:
 
@@ -191,7 +188,7 @@ protected:
  *
  * @code
  * {
- *   Cairo::RefPtr<MyUserFont> face = MyUserFont::create();
+ *   auto face = MyUserFont::create();
  *   cr->set_font_face(face);
  * }  // scope for demonstration purposes
  *
@@ -207,11 +204,11 @@ protected:
  *
  * @since 1.8
  */
-class UserFontFace : public FontFace
+class CAIROMM_API UserFontFace : public FontFace
 {
 public:
 
-  virtual ~UserFontFace();
+  ~UserFontFace() override;
 
   /*
   static RefPtr<UserFontFace> create();
@@ -400,7 +397,7 @@ private:
 // font system support
 #ifdef CAIRO_HAS_FT_FONT
 
-class FtFontFace : public FontFace
+class CAIROMM_API FtFontFace : public FontFace
 {
 public:
   /** Creates a new font face for the FreeType font backend from a pre-opened
@@ -439,6 +436,7 @@ public:
   static RefPtr<FtFontFace> create(FT_Face face, int load_flags);
   //TODO: Add a suitable default value for load_flags?
 
+#ifdef CAIRO_HAS_FC_FONT
   /** Creates a new font face for the FreeType font backend based on a
    * fontconfig pattern. This font can then be used with Context::set_font_face()
    * or FtScaledFont::create().
@@ -468,10 +466,35 @@ public:
    * @since 1.8
    */
   static RefPtr<FtFontFace> create(FcPattern* pattern);
+#endif // CAIRO_HAS_FC_FONT
+
+  /** Sets synthesis options to control how FreeType renders the glyphs for a
+   * particular font face. The given options are ORed with the currently active
+   * options.
+   *
+   * @param synth_flags A set of synthesis options to enable
+   * @since 1.12
+   */
+  void set_synthesize(FtSynthesize synth_flags);
+
+  /** Unsets the specified FreeType glypth synthesis options
+   *
+   * @param synth_flags A set of synthesis options to disable
+   * @since 1.12
+   */
+  void unset_synthesize(FtSynthesize synth_flags);
+
+  /** Returns currently activy FreeType glyph synthesis options
+   *
+   * @since 1.12
+   */
+  FtSynthesize get_synthesize() const;
 
 protected:
   FtFontFace(FT_Face face, int load_flags);
+#ifdef CAIRO_HAS_FC_FONT
   FtFontFace(FcPattern* pattern);
+#endif // CAIRO_HAS_FC_FONT
 };
 
 #endif // CAIRO_HAS_FT_FONT
