@@ -26,13 +26,17 @@
 
 #ifdef nil
 #undef nil
+#include <sigc++/slot.h>
+#define nil NULL
+#else
+#include <sigc++/slot.h>
 #endif
 
 /* end OS X */
 
-#include <sigc++/slot.h>
 #include <cairomm/enums.h>
 #include <cairomm/exception.h>
+#include <cairomm/device.h>
 #include <cairomm/fontoptions.h>
 #include <cairomm/refptr.h>
 
@@ -64,9 +68,14 @@ namespace Cairo
  * different subtypes of cairo surface for different drawing backends.  This
  * class is a base class for all subtypes and should not be used directly
  *
- * Surfaces are reference-counted objects that should be used via Cairo::RefPtr. 
+ * Most surface types allow accessing the surface without using Cairo
+ * functions. If you do this, keep in mind that it is mandatory that you call
+ * Cairo::Surface::flush() before reading from or writing to the surface and that
+ * you must use Cairo::Surface::mark_dirty() after modifying it.
+ *
+ * Surfaces are reference-counted objects that should be used via Cairo::RefPtr.
  */
-class Surface
+class CAIROMM_API Surface
 {
 public:
   /** For example:
@@ -84,6 +93,7 @@ public:
    * @return the status code of the write operation
    */
   typedef sigc::slot<ErrorStatus, const unsigned char* /*data*/, unsigned int /*length*/> SlotWriteFunc;
+
   /**
    * This is the type of function which is called when a backend needs to read
    * data from an input stream. It is passed the buffer to read the data into
@@ -106,8 +116,58 @@ public:
    */
   explicit Surface(cairo_surface_t* cobject, bool has_reference = false);
 
+  Surface(const Surface&) = delete;
+  Surface& operator=(const Surface&) = delete;
+
   virtual ~Surface();
 
+  /**
+   * Return mime data previously attached to surface using the specified mime
+   * type. If no data has been attached with the given mime type then this
+   * returns 0.
+   *
+   * @param mime_type The MIME type of the image data.
+   * @param length This will be set to the length of the image data.
+   * @returns The image data attached to the surface.
+   * @since 1.10
+   */
+  const unsigned char* get_mime_data(const std::string& mime_type, unsigned long& length);
+
+
+  /** For instance,
+   * void on_destroy();
+   */
+  typedef sigc::slot<void> SlotDestroy;
+
+  /** Attach an image in the format mime_type to surface. To remove the data from
+   * a surface, call unset_mime_data() with same mime type.
+   *
+   * The attached image (or filename) data can later be used by backends which
+   * support it (currently: PDF, PS, SVG and Win32 Printing surfaces) to emit
+   * this data instead of making a snapshot of the surface. This approach tends
+   * to be faster and requires less memory and disk space.
+   *
+   * The recognized MIME types are the following: CAIRO_MIME_TYPE_JPEG,
+   * CAIRO_MIME_TYPE_PNG, CAIRO_MIME_TYPE_JP2, CAIRO_MIME_TYPE_URI.
+   *
+   * See corresponding backend surface docs for details about which MIME types
+   * it can handle. Caution: the associated MIME data will be discarded if you
+   * draw on the surface afterwards. Use this function with care.
+   *
+   * @param mime_type The MIME type of the image data. param data The image
+   * @data to attach to the surface. param length The length of the image data.
+   * @param slot_destroy A callback slot that will be called when the Surface
+   *   no longer needs the data. For instance, when the Surface is destroyed or
+   *   when new image data is attached using the same MIME tpe.
+   * @since 1.10
+   */
+  void set_mime_data(const std::string& mime_type, unsigned char* data,
+                     unsigned long length, const SlotDestroy& slot_destroy);
+
+  /** Remove the data from a surface. See set_mime_data().
+   */
+  void unset_mime_data(const std::string& mime_type);
+
   /** Retrieves the default font rendering options for the surface. This allows
    * display surfaces to report the correct subpixel order for rendering on
    * them, print surfaces to disable hinting of metrics and so forth. The
@@ -180,10 +240,34 @@ public:
    */
   void get_device_offset(double& x_offset, double& y_offset) const;
 
-  /** Sets the fallback resolution of the image in dots per inch
+  /**
+   * Set the horizontal and vertical resolution for image fallbacks.
+   *
+   * When certain operations aren't supported natively by a backend, cairo will
+   * fallback by rendering operations to an image and then overlaying that
+   * image onto the output. For backends that are natively vector-oriented,
+   * this function can be used to set the resolution used for these image
+   * fallbacks, (larger values will result in more detailed images, but also
+   * larger file sizes).
+   *
+   * Some examples of natively vector-oriented backends are the ps, pdf, and
+   * svg backends.
+   *
+   * For backends that are natively raster-oriented, image fallbacks are still
+   * possible, but they are always performed at the native device resolution.
+   * So this function has no effect on those backends.
+   *
+   * Note: The fallback resolution only takes effect at the time of completing
+   * a page (with Context::show_page() or Context::copy_page()) so there is
+   * currently no way to have more than one fallback resolution in effect on a
+   * single page.
+   *
+   * The default fallback resoultion is 300 pixels per inch in both dimensions.
    *
    * @param x_pixels_per_inch   Pixels per inch in the x direction
    * @param y_pixels_per_inch   Pixels per inch in the y direction
+   *
+   * @since 1.2
    */
   void set_fallback_resolution(double x_pixels_per_inch, double y_pixels_per_inch);
 
@@ -199,7 +283,8 @@ public:
 
   SurfaceType get_type() const;
 
-  /**This function returns the content type of surface which indicates whether
+  /**
+   * This function returns the content type of surface which indicates whether
    * the surface contains color and/or alpha information.
    *
    * @since 1.8
@@ -266,6 +351,10 @@ public:
 
 #endif // CAIRO_HAS_PNG_FUNCTIONS
 
+  /** This function returns the device for a surface
+   * @return The device for this surface, or an empty RefPtr if the surface has
+   * no associated device */
+  RefPtr<Device> get_device();
 
   /** The underlying C cairo surface type
    */
@@ -298,14 +387,33 @@ public:
    */
   static RefPtr<Surface> create(const RefPtr<Surface> other, Content content, int width, int height);
 
+  /** Create a new surface that is a rectangle within the target surface. All
+   * operations drawn to this surface are then clipped and translated onto the
+   * target surface. Nothing drawn via this sub-surface outside of its bounds is
+   * drawn onto the target surface, making this a useful method for passing
+   * constrained child surfaces to library routines that draw directly onto the
+   * parent surface, i.e. with no further backend allocations, double buffering
+   * or copies.
+   *
+   * @Note The semantics of subsurfaces have not been finalized yet unless the
+   * rectangle is in full device units, is contained within the extents of the
+   * target surface, and the target or subsurface's device transforms are not
+   * changed.
+   *
+   * @param target an existing surface for which the sub-surface will point to
+   * @param x the x-origin of the sub-surface from the top-left of the target surface (in device-space units)
+   * @param y the y-origin of the sub-surface from the top-left of the target surface (in device-space units)
+   * @param width width of the sub-surface (in device-space units)
+   * @param height height of the sub-surface (in device-space units)
+   *
+   * @since 1.10
+   */
+  static RefPtr<Surface> create(const RefPtr<Surface>& target, double x, double y, double width, double height);
+
 protected:
   /** The underlying C cairo surface type that is wrapped by this Surface
    */
   cobject* m_cobject;
-
-private:
-  Surface(const Surface&);
-  Surface& operator=(const Surface&);
 };
 
 /** @example image-surface.cc
@@ -319,16 +427,16 @@ private:
  * An ImageSurface is the most generic type of Surface and the only one that is
  * available by default.  You can either create an ImageSurface whose data is
  * managed by Cairo, or you can create an ImageSurface with a data buffer that
- * you allocated yourself so that you can have full access to the data.  
+ * you allocated yourself so that you can have full access to the data.
  *
  * When you create an ImageSurface with your own data buffer, you are free to
  * examine the results at any point and do whatever you want with it.  Note that
  * if you modify anything and later want to continue to draw to the surface
- * with cairo, you must let cairo know via Cairo::Surface::mark_dirty() 
+ * with cairo, you must let cairo know via Cairo::Surface::mark_dirty()
  *
  * Note that like all surfaces, an ImageSurface is a reference-counted object that should be used via Cairo::RefPtr.
  */
-class ImageSurface : public Surface
+class CAIROMM_API ImageSurface : public Surface
 {
 protected:
   //TODO?: Surface(cairo_surface_t *target);
@@ -343,7 +451,7 @@ public:
    */
   explicit ImageSurface(cairo_surface_t* cobject, bool has_reference = false);
 
-  virtual ~ImageSurface();
+  ~ImageSurface() override;
 
   /** Gets the width of the ImageSurface in pixels
    */
@@ -353,32 +461,39 @@ public:
    */
   int get_height() const;
 
+  /// @{
   /**
    * Get a pointer to the data of the image surface, for direct
    * inspection or modification.
    *
    * Return value: a pointer to the image data of this surface or NULL
    * if @surface is not an image surface.
+   *
+   * @since 1.2
    */
   unsigned char* get_data();
   const unsigned char* get_data() const;
+  /// @}
 
-  /** gets the format of the surface
+  /**
+   * Gets the format of the surface
+   * @since 1.2
    */
   Format get_format() const;
 
   /**
-   * Return value: the stride of the image surface in bytes (or 0 if
-   * @surface is not an image surface). The stride is the distance in
-   * bytes from the beginning of one row of the image data to the
-   * beginning of the next row.
+   * Returns the stride of the image surface in bytes (or 0 if surface is not
+   * an image surface). The stride is the distance in bytes from the beginning
+   * of one row of the image data to the beginning of the next row.
+   *
+   * @since 1.2
    */
   int get_stride() const;
 
   /**
-   * This function provides a stride value that will respect all
-   * alignment requirements of the accelerated image-rendering code
-   * within cairo. Typical usage will be of the form:
+   * This function provides a stride value that will respect all alignment
+   * requirements of the accelerated image-rendering code within cairo. Typical
+   * usage will be of the form:
    *
    * @code
    * int stride;
@@ -397,7 +512,7 @@ public:
    *
    * @since 1.6
    **/
-  static int format_stride_for_width (Cairo::Format format, int width);
+  static int format_stride_for_width(Cairo::Format format, int width);
 
   /**
    * Creates an image surface of the specified format and dimensions. Initially
@@ -469,12 +584,119 @@ public:
 };
 
 
+/**
+ * A recording surface is a surface that records all drawing operations at the
+ * highest level of the surface backend interface, (that is, the level of paint,
+ * mask, stroke, fill, and show_text_glyphs). The recording surface can then be
+ * "replayed" against any target surface by using it as a source surface.
+ *
+ * If you want to replay a surface so that the results in `target` will be
+ * identical to the results that would have been obtained if the original
+ * operations applied to the recording surface had instead been applied to the
+ * target surface, you can use code like this:
+ *
+ *     Cairo::RefPtr<Cairo::Context> context = Cairo::Context::create(target);
+ *     context->set_source(recording_surface, 0.0, 0.0);
+ *     context->paint();
+ *
+ * A recording surface is logically unbounded, i.e. it has no implicit
+ * constraint on the size of the drawing surface. However, in practice this is
+ * rarely useful as you wish to replay against a particular target surface with
+ * known bounds. For this case, it is more efficient to specify the target
+ * extents to the recording surface upon creation.
+ *
+ * The recording phase of the recording surface is careful to snapshot all
+ * necessary objects (paths, patterns, etc.), in order to achieve accurate
+ * replay.
+ *
+ * Note that like all surfaces, a RecordingSurface is a reference-counted object
+ * that should be used via Cairo::RefPtr.
+ */
+class CAIROMM_API RecordingSurface : public Surface
+{
+public:
+
+  /**
+   * Create a C++ wrapper for the C instance. This C++ instance should then be
+   * given to a RefPtr.
+   * @param cobject The C instance.
+   * @param has_reference Whether we already have a reference. Otherwise, the
+   * constructor will take an extra reference.
+   */
+  explicit RecordingSurface(cairo_surface_t* cobject, bool has_reference = false);
+
+  virtual ~RecordingSurface();
+
+  /**
+   * Measures the extents of the operations stored within the recording
+   * surface.  This is useful to compute the required size of an image surface
+   * (or equivalent) into which to replay the full sequence of drawing
+   * operations.
+   * @return a Rectangle with `x` and `y` set to the coordinates of the top-left
+   * of the ink bounding box, and `width` and `height` set to the width and
+   * height of the ink bounding box.
+   */
+  Rectangle ink_extents() const;
+
+  /**
+   * Get the extents of the recording surface, if the surface is bounded.
+   * @param extents the Rectangle in which to store the extents if the recording
+   * surface is bounded
+   * @return true if the recording surface is bounded, false if the recording
+   * surface is unbounded (in which case `extents` will not be set).
+   */
+  bool get_extents(Rectangle& extents) const;
+
+  /**
+   * Creates a recording surface which can be used to record all drawing
+   * operations at the highest level (that is, the level of paint, mask, stroke,
+   * fill and show_text_glyphs). The recording surface can then be "replayed"
+   * against any target surface by using it as a source to drawing operations.
+   * The recording surface will be unbounded.
+   *
+   * The recording phase of the recording surface is careful to snapshot all
+   * necessary objects (paths, patterns, etc.), in order to achieve accurate
+   * replay.
+   *
+   * @param content the content of the recording surface; defaults to
+   * Cairo::Content::CONTENT_COLOR_ALPHA.
+   * @return a RefPtr to the newly created recording surface.
+   */
+  static RefPtr<RecordingSurface> create(Content content = Content::CONTENT_COLOR_ALPHA);
+
+  /**
+   * Creates a recording surface which can be used to record all drawing
+   * operations at the highest level (that is, the level of paint, mask, stroke,
+   * fill and show_text_glyphs). The recording surface can then be "replayed"
+   * against any target surface by using it as a source to drawing operations.
+   * The recording surface will be bounded by the given rectangle.
+   *
+   * The recording phase of the recording surface is careful to snapshot all
+   * necessary objects (paths, patterns, etc.), in order to achieve accurate
+   * replay.
+   *
+   * @param	extents the extents to record
+   * @param	content the content of the recording surface; defaults to
+   * Cairo::Content::CONTENT_COLOR_ALPHA.
+   * @return	a RefPtr to the newly created recording surface.
+   */
+  static RefPtr<RecordingSurface> create(const Rectangle& extents, Content content = Content::CONTENT_COLOR_ALPHA);
+
+};
+
+
 #ifdef CAIRO_HAS_PDF_SURFACE
 
 /** @example pdf-surface.cc
  * An example of using Cairo::PdfSurface class to render to PDF
  */
 
+typedef enum
+{
+  PDF_VERSION_1_4 = CAIRO_PDF_VERSION_1_4,
+  PDF_VERSION_1_5 = CAIRO_PDF_VERSION_1_5
+} PdfVersion;
+
 /** A PdfSurface provides a way to render PDF documents from cairo.  This
  * surface is not rendered to the screen but instead renders the drawing to a
  * PDF file on disk.
@@ -482,7 +704,7 @@ public:
  * @note For this Surface to be available, cairo must have been compiled with
  * PDF support
  */
-class PdfSurface : public Surface
+class CAIROMM_API PdfSurface : public Surface
 {
 public:
 
@@ -494,7 +716,7 @@ public:
    * constructor will take an extra reference.
    */
   explicit PdfSurface(cairo_surface_t* cobject, bool has_reference = false);
-  virtual ~PdfSurface();
+  ~PdfSurface() override;
 
   /** Creates a PdfSurface with a specified dimensions that will be saved as
    * the given filename
@@ -502,6 +724,7 @@ public:
    * @param filename    The name of the PDF file to save the surface to
    * @param width_in_points   The width of the PDF document in points
    * @param height_in_points   The height of the PDF document in points
+   * @since 1.2
    */
   static RefPtr<PdfSurface> create(std::string filename, double width_in_points, double height_in_points);
 
@@ -512,7 +735,6 @@ public:
    * write data to an output stream
    * @param width_in_points   The width of the PDF document in points
    * @param height_in_points   The height of the PDF document in points
-   *
    * @since 1.8
    */
   static RefPtr<PdfSurface> create_for_stream(const SlotWriteFunc& write_func, double width_in_points, double height_in_points);
@@ -532,6 +754,32 @@ public:
  **/
   void set_size(double width_in_points, double height_in_points);
 
+  /**
+   * Restricts the generated PDF file to version. See get_versions() for a list
+   * of available version values that can be used here.
+   *
+   * This function should only be called before any drawing operations have been
+   * performed on the given surface. The simplest way to do this is to call this
+   * function immediately after creating the surface.
+   *
+   * @since 1.10
+   */
+  void restrict_to_version(PdfVersion version);
+
+  /** Retrieves the list of PDF versions supported by cairo. See
+   * restrict_to_version().
+   *
+   * @since 1.10
+   */
+  static const std::vector<PdfVersion> get_versions();
+
+  /** Get the string representation of the given version id. This function will
+   * return an empty string if version isn't valid. See get_versions()
+   * for a way to get the list of valid version ids.
+   *
+   * @since 1.10
+   */
+  static std::string version_to_string(PdfVersion version);
 };
 
 #endif  // CAIRO_HAS_PDF_SURFACE
@@ -559,7 +807,7 @@ typedef enum {
  * @note For this Surface to be available, cairo must have been compiled with
  * PostScript support
  */
-class PsSurface : public Surface
+class CAIROMM_API PsSurface : public Surface
 {
 public:
 
@@ -571,7 +819,7 @@ public:
    * constructor will take an extra reference.
    */
   explicit PsSurface(cairo_surface_t* cobject, bool has_reference = false);
-  virtual ~PsSurface();
+  ~PsSurface() override;
 
   /** Creates a PsSurface with a specified dimensions that will be saved as the
    * given filename
@@ -714,7 +962,7 @@ typedef enum
  * @note For this Surface to be available, cairo must have been compiled with
  * SVG support
  */
-class SvgSurface : public Surface
+class CAIROMM_API SvgSurface : public Surface
 {
 public:
 
@@ -726,7 +974,7 @@ public:
    * constructor will take an extra reference.
    */
   explicit SvgSurface(cairo_surface_t* cobject, bool has_reference = false);
-  virtual ~SvgSurface();
+  ~SvgSurface() override;
 
 
   /** Creates a SvgSurface with a specified dimensions that will be saved as the
@@ -753,7 +1001,7 @@ public:
   /** @deprecated Use SvgSurface::create_for_stream() instead */
   static RefPtr<SvgSurface> create(cairo_write_func_t write_func, void *closure, double width_in_points, double height_in_points);
 
-  /** 
+  /**
    * Restricts the generated SVG file to the given version. See get_versions()
    * for a list of available version values that can be used here.
    *
@@ -767,7 +1015,7 @@ public:
 
   /** Retrieves the list of SVG versions supported by cairo. See
    * restrict_to_version().
-   * 
+   *
    * @since 1.2
    */
   static const std::vector<SvgVersion> get_versions();
@@ -801,7 +1049,7 @@ public:
  * @warning This is an experimental surface.  It is not yet marked as a fully
  * supported surface by the cairo library
  */
-class GlitzSurface : public Surface
+class CAIROMM_API GlitzSurface : public Surface
 {
 
 public:
@@ -815,7 +1063,7 @@ public:
    */
   explicit GlitzSurface(cairo_surface_t* cobject, bool has_reference = false);
 
-  virtual ~GlitzSurface();
+  ~GlitzSurface() override;
 
   /** Creates a new GlitzSurface
    *
